﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;


namespace LocalJoost.Models
{
    public static class BindingHelpers
    {
        #region MapTileSourceProperty
        // Name, Property type, type of object that hosts the property, method to call when anything changes
        public static readonly DependencyProperty MapTileSourceProperty =
            DependencyProperty.RegisterAttached("MapTileSource", typeof(MultiScaleTileSource),
            typeof(BindingHelpers), new PropertyMetadata(SetMapTileSourceCallback));

        // Called when MapTileSource is retrieved
        public static MultiScaleTileSource GetMapTileSource(DependencyObject obj)
        {
            return obj.GetValue(MapTileSourceProperty) as MultiScaleTileSource;
        }

        // Called when MapTileSource is set
        public static void SetMapTileSource(DependencyObject obj, MultiScaleTileSource value)
        {
            obj.SetValue(MapTileSourceProperty, value);
        }

        private static void SetMapTileSourceCallback(object sender, DependencyPropertyChangedEventArgs args)
        {
            var c = sender as MultiScaleImage;
            if (c != null)
            {
                c.Source = args.NewValue as MultiScaleTileSource;
            }
        }
        #endregion

        #region ZoomEventPointProperty
        public static readonly DependencyProperty ZoomEventPointProperty =
            DependencyProperty.RegisterAttached("ZoomEventPoint", typeof(ZoomEventPoint),
            typeof(BindingHelpers), new PropertyMetadata(SetZoomEventPointCallback));

        public static ZoomEventPoint GetZoomEventPoint(DependencyObject obj)
        {
            return (ZoomEventPoint)obj.GetValue(ZoomEventPointProperty);
        }

        public static void SetZoomEventPoint(DependencyObject obj, ZoomEventPoint value)
        {
            obj.SetValue(ZoomEventPointProperty, value);
        }

        private static void SetZoomEventPointCallback(object sender, DependencyPropertyChangedEventArgs args)
        {
            var msi = sender as MultiScaleImage;
            var ep = (ZoomEventPoint)args.NewValue;
            if (msi != null)
            {
                var zoomEventPoint = msi.ElementToLogicalPoint(ep.Event.GetPosition(msi));
                msi.ZoomAboutLogicalPoint(ep.ZoomFactor, zoomEventPoint.X, zoomEventPoint.Y);
            }
        }

        #endregion

        #region Dragging properties

        #region StartDragEventDataProperty

        public static readonly DependencyProperty StartDragEventDataProperty =
            DependencyProperty.RegisterAttached("StartDragEventData",
            typeof(MouseEventArgs),
            typeof(BindingHelpers),
            new PropertyMetadata(SetStartDragEventDataCallback));

        public static MouseEventArgs GetStartDragEventData(DependencyObject obj)
        {
            return (MouseEventArgs)obj.GetValue(StartDragEventDataProperty);
        }

        public static void SetStartDragEventData(DependencyObject obj, MouseEventArgs value)
        {
            obj.SetValue(StartDragEventDataProperty, value);
        }

        private static void SetStartDragEventDataCallback(object sender,
            DependencyPropertyChangedEventArgs args)
        {
            var msi = sender as MultiScaleImage;
            var e = args.NewValue as MouseEventArgs;
            if (msi != null && e != null)
            {
               
                SetDragData(msi, new DragData
                {
                    DragOffset = e.GetPosition(msi),
                    CurrentPosition = msi.ViewportOrigin
                });
            }
        }

        #endregion

        #region DragDataProperty

        public static readonly DependencyProperty DragDataProperty =
            DependencyProperty.RegisterAttached("DragData", typeof(DragData),
            typeof(BindingHelpers), null);

        public static DragData GetDragData(DependencyObject obj)
        {
            return (DragData)obj.GetValue(DragDataProperty);
        }

        public static void SetDragData(DependencyObject obj, DragData value)
        {
            obj.SetValue(DragDataProperty, value);
        }

        #endregion

        #region MoveEventDataProperty

        public static readonly DependencyProperty MoveEventDataProperty =
             DependencyProperty.RegisterAttached("MoveEventData",
             typeof(MouseEventArgs),
             typeof(BindingHelpers),
             new PropertyMetadata(SetMoveEventDataCallback));

        public static MouseEventArgs GetMoveEventData(DependencyObject obj)
        {
            return (MouseEventArgs)obj.GetValue(MoveEventDataProperty);
        }

        public static void SetMoveEventData(DependencyObject obj, MouseEventArgs value)
        {
            obj.SetValue(MoveEventDataProperty, value);
        }

        private static void SetMoveEventDataCallback(object sender,
            DependencyPropertyChangedEventArgs args)
        {

            var msi = sender as MultiScaleImage;
            var e = args.NewValue as MouseEventArgs;
            if (msi != null && e != null)
            {
                var dragData = GetDragData(msi);
                var newOrigin =
                    new Point
                    {
                        X = dragData.CurrentPosition.X -
                            (((e.GetPosition(msi).X - dragData.DragOffset.X) / 
                              msi.ActualWidth) * msi.ViewportWidth),
                        Y = dragData.CurrentPosition.Y -
                            (((e.GetPosition(msi).Y - dragData.DragOffset.Y) / 
                              msi.ActualHeight) * msi.ViewportWidth)
                    };
                msi.ViewportOrigin = newOrigin;
            }
        }
        #endregion

        #endregion
    }
}
