using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using LocalJoost.TileSource;
using System.Collections.Generic;
using GalaSoft.MvvmLight.Command;

namespace LocalJoost.Models
{
  public class MapViewModel : INotifyPropertyChanged
  {
    public MapViewModel()
    {
      // Initialize Map sources
      _allTileSources = new LinkedList<MapViewItemModel>(new[]{
        new MapViewItemModel{ MapName="OSM: OsmaRender", 
          TileSource = new OsmaRenderSource()},
        new MapViewItemModel{ MapName="OSM: Mapnik", 
          TileSource = new MapnikSource()},
        new MapViewItemModel{ MapName="Google Street", 
          TileSource = new GoogleTileSource(GoogleTileSourceType.Street)},
        new MapViewItemModel{ MapName="Google Satellite", 
          TileSource = new GoogleTileSource(GoogleTileSourceType.Satellite)},
        new MapViewItemModel{ MapName="Google Hybrid", 
          TileSource = new GoogleTileSource(GoogleTileSourceType.Hybrid)},
        new MapViewItemModel{ MapName="Bing Road", 
          TileSource = new VERoadTileSource()},
        new MapViewItemModel{ MapName="Bing Aerial", 
          TileSource = new VEAerialTileSource()}
      });

      // Initialize text and zoomfactor for zoom in/out toggle
      _zoomactions = new List<ZoomActionDescription>
          {
            new ZoomActionDescription 
             {Description = "Double tap zooms in", ZoomFactor = 2.0},
            new ZoomActionDescription 
             {Description = "Double tap zooms out", ZoomFactor = 0.5}
          };

      CurrentTileSource = _allTileSources.First.Value;
      ZoomAction = _zoomactions[0];
      _lastMouseDown = DateTime.Now;
    }

    #region Maps
    private readonly LinkedList<MapViewItemModel> _allTileSources;
    private readonly List<ZoomActionDescription> _zoomactions;

    /// <summary>
    /// Called by the Windows Phone client
    /// when the user swipes over the map tile
    /// </summary>
    public ICommand SetNextMapCommandWp7
    {
        get
        {
            return new RelayCommand<ManipulationCompletedEventArgs>(SetNextMapWp7);
        }
    }

    /// <summary>
    /// Determine to select previous or next map
    /// </summary>
    /// <param name="e"></param>
    public void SetNextMapWp7(ManipulationCompletedEventArgs e)
    {
        if (e.TotalManipulation.Translation.X > 0)
        {
            SetPreviousMap();
        }
        else
        {
            SetNextMap();
        }
    }

    /// <summary>
    /// Move to the next map
    /// </summary>
    public void SetNextMap()
    {
      var currentNode = _allTileSources.Find(CurrentTileSource);
      if (currentNode != null)
      {
        CurrentTileSource =
          currentNode.Next != null ? currentNode.Next.Value : _allTileSources.First.Value;
      }
    }

    /// <summary>
    /// Move to the previous map
    /// </summary>
    public void SetPreviousMap()
    {
      var currentNode = _allTileSources.Find(CurrentTileSource);
      if (currentNode != null)
      {
        CurrentTileSource =
          currentNode.Previous != null ? currentNode.Previous.Value : _allTileSources.Last.Value;
      }
    }

    /// <summary>
    /// Command called by Silverlight 4 client
    /// when user clicks on map title
    /// </summary>
    public ICommand SetNextMapCommand
    {
        get
        {
            return new RelayCommand(SetNextMap);
        }
    }



    private MapViewItemModel _tileSource;
    public MapViewItemModel CurrentTileSource
    {
      get
      {
        return _tileSource;
      }
      private set
      {
        _tileSource = value;
        NotifyPropertyChanged("CurrentTileSource");
      }
    }
    #endregion


    public ICommand ToggleZoomCommand
    {
      get
      {
        return new RelayCommand(() =>
        {
          ZoomAction =
            ZoomAction.Equals(_zoomactions[0]) ? _zoomactions[1] : _zoomactions[0];
        });
      }
    }

    private DateTime _lastMouseDown;
    private readonly TimeSpan _timeSpanForDoubleClick =
      new TimeSpan(0, 0, 0, 0, 250);

    public ICommand MouseLeftButtonDown
    {
      get
      {
        return new RelayCommand<MouseEventArgs>(e =>
        {

          var t = DateTime.Now - _lastMouseDown;
          if (t < _timeSpanForDoubleClick)
          {
            // assume double click - Zoom
            ZoomEventPoint = new ZoomEventPoint { Event = e, ZoomFactor = ZoomAction.ZoomFactor };
          }
          else
          {
            // Assume start of dragging 
            StartMoveEvent = e;
          }
          _lastMouseDown = DateTime.Now;
        });
      }
    }

    #region Dragging
    public ICommand MouseLeftButtonUp
    {
      get
      {
        return new RelayCommand<MouseEventArgs>(e =>
        {
          // Stop of dragging 
          StartMoveEvent = null;
        });
      }
    }


    public ICommand MouseMove
    {
      get
      {
        return new RelayCommand<MouseEventArgs>(e =>
        {
          if (StartMoveEvent != null)
          {
            EndMoveEvent = e;
          }
        });
      }
    }

    private MouseEventArgs _startMoveEvent;
    public MouseEventArgs StartMoveEvent
    {
      get
      {
        return _startMoveEvent;
      }
      set
      {
        _startMoveEvent = value;
        NotifyPropertyChanged("StartMoveEvent");
      }
    }

    private MouseEventArgs _endMoveEvent;
    public MouseEventArgs EndMoveEvent
    {
      get
      {
        return _endMoveEvent;
      }
      set
      {
        _endMoveEvent = value;
        NotifyPropertyChanged("EndMoveEvent");
      }
    }
    #endregion


    private ZoomEventPoint _zoomEventPoint;
    public ZoomEventPoint ZoomEventPoint
    {
      get
      {
        return _zoomEventPoint;
      }
      private set
      {
        _zoomEventPoint = value;
        NotifyPropertyChanged("ZoomEventPoint");
      }
    }

    private Point _origin;
    public Point ViewportOrigin
    {
      get
      {
        return _origin;
      }
      set
      {
        _origin = value;
        NotifyPropertyChanged("ViewportOrigin");
      }
    }

    private ZoomActionDescription _zoomAction;
    public ZoomActionDescription ZoomAction
    {
      get
      {
        return _zoomAction;
      }
      set
      {
        _zoomAction = value;
        NotifyPropertyChanged("ZoomAction");
      }
    }

    public event PropertyChangedEventHandler PropertyChanged;
    private void NotifyPropertyChanged(String propertyName)
    {

      if (null != PropertyChanged)
      {
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }

  }
}